#include "math.h"
#include "F16_AeroData_cfnc.c"              /* LOFI Look-up header file*/

#define pi 3.14159
#define g    32.17   // gravity, ft/s^2 
#define m    636.94  // mass, slugs 
#define B    30.0    // span, ft 
#define S    300.0   // planform area, ft^2 
#define cbar 11.32   // mean aero chord, ft 
#define xcgr 0.35    // reference center of gravity as a fraction of cbar 
#define xcg  0.30    // center of gravity as a fraction of cbar. 
#define Heng 0.0     // turbine momentum along roll axis. 
#define ail_bw  20.0  // aileron bandwidth, correct value is 20.2Hz
#define rud_bw  20.0  // rudder bandwidth, correct value is 20.2Hz

#define Jy   55814.0  // slug-ft^2  
#define Jxz  982.0    // slug-ft^2      
#define Jz   63100.0  // slug-ft^2 
#define Jx   9496.0   // slug-ft^2 

void aerodata(double *xu, double *f, void *user_data){
        
    double *temp;
    double r2d  = 180.0/pi; // radians to degrees 

    // longitudinal states 
    double t = xu[0];           // time
    double x_beta = xu[1];      // sideslip (state)
    double x_phi =  xu[2];      // roll (state)
    double x_psi =  xu[3];      // yaw (state)
    double x_P =    xu[4];      // roll rate (state) 
    double x_R =    xu[5];      // total speed (state)
    double u_ail =  xu[6];      // aileron position (state) 
    double u_ail_dot =  xu[7];  // aileron deflection rate (state)
    double u_rud =      xu[8];  // rudder position (state) 
    double u_rud_dot =  xu[9];  // rudder deflection rate (state)
    double uC_ail = xu[10];     // aileron deflection cmd (input)    
    double uC_rud = xu[11];     // rudder (input) 
    double p_qbar = xu[12];     // dynamic pressure (parameter)
    double p_alpha =xu[13];     // constant AoA (parameter)
    double p_Vt =   xu[14];     // constant speed (parameter)
    double p_theta =xu[15];     // constant pitch (parameter)
    double p_ft =   xu[16];     // constant thrust (parameter)
    
    // intermediate longitudinal states 
    double Cy_tot;  // total force coefficient in Y
    double Cy;      // force coefficient in Y
    double dYdR, dYdP;
    double Cyr;     // force coefficient in Y due to r
    double Cyp;     // force coefficient in Y due to p
    double Cn_tot;  // total moment coefficient
    double Cn;      // yaw moment coefficient 
    double N_tot;   // total yaw moment
    double dNdR, dNdP;
    double Cnr;     // yaw moment coefficient due to r
    double Cnp;     // yaw moment coefficient due to p
    double Cl_tot;  // total moment coefficient
    double Cl;      // roll moment coefficient 
    double L_tot;   // total roll moment
    double dLdR, dLdP;
    double Clr;     // roll moment coefficient due to r
    double Clp;     // roll moment coefficient due to p
    double g2;      // gravity component
    double C;       // cross-wind force       
    double Y_tot;   // total force in Y       
    double Cx;      // force coefficient in X
    double Cx_tot;  // total force coefficient in X 
    double X_tot;   // total force in X 
    double Cz;      // force coefficient in Z
    double Cz_tot;  // total force coefficient in Z 
    double Z_tot;   // total force in Z           
    double dail;    // aileron normalized against max angle
    double drud;    // rudder normalized against max angle
    double denom;   // denominator for P_dot and R_dot calculations
    double dLdail, dLdrud; 
    double delta_Cl_a20;
    double delta_Cl_r30;
    double dNdail, dNdrud; 
    double delta_Cn_a20;
    double delta_Cn_r30;
    

    
    // temporary memory allocation
    temp = (double *)malloc(9*sizeof(double));  /*size of 9.1 array*/

    /* %%%%%%%%%%%%%%%%%%
          Table lookup
       %%%%%%%%%%%%%%%%%% */    

    damping(p_alpha*r2d,temp);
        //Cxq = temp[0];
        Cyr = temp[1];
        Cyp = temp[2];
        //Czq = temp[3];
        Clr = temp[4];
        Clp = temp[5];
        //Cmq = temp[6];
        Cnr = temp[7];
        Cnp = temp[8];

    dmomdcon(p_alpha*r2d,x_beta*r2d,temp);
        delta_Cl_a20 = temp[0];     
        delta_Cl_r30 = temp[1];     
        delta_Cn_a20 = temp[2];     
        delta_Cn_r30 = temp[3];     
        
//     cxcm(p_alpha*r2d,p_el,temp);
//         Cx = temp[0];
//         // Cm = temp[1];

    clcn(p_alpha*r2d,x_beta*r2d,temp);
        Cl = temp[0];
        Cn = temp[1];

    // control surfaces normalized against their max angle
    dail  = u_ail/21.5;
    drud  = u_rud/30.0;  

    Cy = -.02*x_beta*r2d + .021*dail + .086*drud;

//     cz(p_alpha,x_beta,p_el,temp);
//     Cz = temp[0];
    
    free(temp); 
        
    // Intermediate states

    // Cy_tot
    dYdR = (B/(2*p_Vt))*Cyr;
    dYdP = (B/(2*p_Vt))*Cyp;
    Cy_tot = Cy + dYdR*x_R + dYdP*x_P;

    // Cn_tot 
    dNdR = (B/(2*p_Vt))*Cnr;
    dNdP = (B/(2*p_Vt))*Cnp;
    dNdail = delta_Cn_a20;
    dNdrud = delta_Cn_r30;
    Cn_tot = Cn - Cy_tot*(xcgr-xcg)*(cbar/B) + dNdail*dail + dNdrud*drud + dNdR*x_R + dNdP*x_P;

   // Cl_tot
    dLdR = (B/(2*p_Vt))*Clr;
    dLdP = (B/(2*p_Vt))*Clp;
    dLdail = delta_Cl_a20;
    dLdrud = delta_Cl_r30;
    Cl_tot = Cl + dLdail*dail + dLdrud*drud + dLdR*x_R + dLdP*x_P;
    
    // total moments from coefficients
    L_tot = Cl_tot*p_qbar*S*B;   
    N_tot = Cn_tot*p_qbar*S*B;

    // total lateral force from coefficients 
    Y_tot = p_qbar*S*Cy_tot;

    // denominator for P_dot and R_dot calculations
    denom = Jx*Jz - Jxz*Jxz;

//     // some 'constant' longitudinal coupling effects
//     // Cx_tot
//     Cx_tot = Cx; 

//     // Cz_tot
//     Cz_tot = Cz; 
    
    // total coupling forces from long. coefficients
//     X_tot = p_qbar*S*Cx_tot;  
//     Z_tot = p_qbar*S*Cz_tot;  
    X_tot = 0.0; // no coupling effect assumed!
    Z_tot = 0.0; // no coupling effect assumed!
    
    // cross-wind force C, longitudinal components are constant
    C = -X_tot*cos(p_alpha)*sin(x_beta)+Y_tot*cos(x_beta)-Z_tot*sin(p_alpha)*sin(x_beta);

    // gravity component
    g2 = g*(cos(p_alpha)*sin(x_beta)*sin(p_theta) + cos(x_beta)*sin(x_phi)*cos(p_theta)-sin(p_alpha)*sin(x_beta)*cos(x_phi)*cos(p_theta));    
        
    
    // Differential Equations      
                 
    // psi_dot
    f[2] = x_R*cos(x_phi)/cos(p_theta);
            
   
} /*##### END of aerodata() ####*/



/*########################################*/
/*########################################*/



