#include "math.h"
#include "F16_AeroData_cfnc.c"              /* LOFI Look-up header file*/

#define pi 3.14159
#define g    32.17   // gravity, ft/s^2 
#define m    636.94  // mass, slugs 
#define B    30.0    // span, ft 
#define S    300.0   // planform area, ft^2 
#define cbar 11.32   // mean aero chord, ft 
#define xcgr 0.35    // reference center of gravity as a fraction of cbar 
#define xcg  0.30    // center of gravity as a fraction of cbar. 
#define Heng 0.0     // turbine momentum along roll axis. 
#define el_bw  20.0  // elevator bandwidth, correct value is 20.2Hz

#define Jy   55814.0  // slug-ft^2  
#define Jxz  982.0    // slug-ft^2      
#define Jz   63100.0  // slug-ft^2 
#define Jx   9496.0   // slug-ft^2 

void aerodata(double *xu, double *f, void *user_data){
        
    double *temp;
    double beta = 0.0;      // only longitudinal dynamics considered
    double r2d  = 180.0/pi; // radians to degrees 

    // longitudinal states 
    double t = xu[0];       // time
//     double x_h = xu[1];     // altitude (state)
    double x_gamma = xu[1]; // flight path (state)
    double x_Q = xu[2];     // pitch rate (state)
    double x_alpha = xu[3]; // AoA (state) in rad 
    double x_Vt = xu[4];    // total speed (state)
    double u_el = xu[5];    // elevator position (state) 
    double u_el_dot = xu[6];// elevator deflection rate (state)
    double uC_el = xu[7];   // elevator deflection cmd (input)    
    double uC_ft = xu[8];   // thrust (input) 
    double p_qbar = xu[9]; // dynamic pressure (parameter)
    
    // intermediate longitudinal states 
    double Czq;     // force coefficient in Z due to q
    double Cz;      // force coefficient in Z  
    double dZdQ;       
    double Cz_tot;  // total force coefficient in Z       
    double Cmq;     // moment coefficient due to q  
    double Cm;      // moment coefficient 
    double dMdQ;
    double Cm_tot;  // total moment coefficient
    double M_tot;   // total moment
    double Cxq;     // force coefficient in X due to q
    double Cx;      // force coefficient in X
    double dXdQ;
    double Cx_tot;  // total force coefficient in X 
    double X_tot;   // total force in X       
    double Z_tot;   // total force in Z           
    double D;       // drag force       
    double L;       // lift force    
    double U;       // velocity in X    
    double W;       // velocity in Z

    // temporary memory allocation
    temp = (double *)malloc(9*sizeof(double));  /*size of 9.1 array*/

    /* %%%%%%%%%%%%%%%%%%
          Table lookup
       %%%%%%%%%%%%%%%%%% */    

    damping(x_alpha*r2d,temp);
        Cxq = temp[0];
        //Cyr = temp[1];
        //Cyp = temp[2];
        Czq = temp[3];
        //Clr = temp[4];
        //Clp = temp[5];
        Cmq = temp[6];
        //Cnr = temp[7];
        //Cnp = temp[8];

    cxcm(x_alpha*r2d,u_el,temp);
        Cx = temp[0];
        Cm = temp[1];

    cz(x_alpha*r2d,beta,u_el,temp);
        Cz = temp[0];
                            
    free(temp); 
        
    // Intermediate states
        
    // Cz_tot
    dZdQ = (cbar/(2*x_Vt))*Czq;
    Cz_tot = Cz + dZdQ*x_Q;

    // Cm_tot
    dMdQ = (cbar/(2*x_Vt))*Cmq;
    Cm_tot = Cm + Cz_tot*(xcgr-xcg) + dMdQ*x_Q; //% replace with x_Q ??
     
    // total moments from coefficients, M_tot 
    M_tot = p_qbar*S*cbar*Cm_tot;
   
    // Cx_tot
    dXdQ = (cbar/(2*x_Vt))*Cxq;
    Cx_tot = Cx + dXdQ*x_Q; //% replace with x_Q ??

    // total longitudinal forces from coefficients, X_tot and Z_tot
    X_tot = p_qbar*S*Cx_tot;
    Z_tot = p_qbar*S*Cz_tot;    
 
    // drag D, lateral components neglected!
    D = -X_tot*cos(x_alpha)-Z_tot*sin(x_alpha);    
    // lift L, lateral components neglected!
    L = X_tot*sin(x_alpha)-Z_tot*cos(x_alpha);
    
    // calculate U and W from the wind axes longitudinal speeds
//     U = x_Vt*cos(x_alpha);
//     W = x_Vt*sin(x_alpha);
    
    
    // Differential Equations    

//     f[0] = U*sin(x_gamma+x_alpha) - W*cos(x_gamma+x_alpha); // h_dot

    f[0] = (1/(m*x_Vt))*(uC_ft*sin(x_alpha)+L-m*g*cos(x_gamma));  //gamma_dot
    f[1] = M_tot/Jy;   // q_dot
        
    f[2] = x_Q-f[0];  //alpha_dot 
    f[3] = (1/m)*(uC_ft*cos(x_alpha)-D-m*g*sin(x_gamma)); //Vt_dot
        
    // option: include actuator dynamics
    f[4] = el_bw*(uC_el - u_el); // el_dot, first order actuator dynamics   
    u_el_dot = f[4];    // u_el_dot is a state!
    f[5] = 0;           // u_el_ddot, zero accel, constant elevator speed
     
   
} /*##### END of aerodata() ####*/



/*########################################*/
/*########################################*/



