%--------------------------------------------------------------------------
% file/function name: altitude_NMPC %
%--------------------------------------------------------------------------

% clear;
BEGIN_ACADO; 
    
    % Acado function name property
    acadoSet('problemname', 'altitude_NMPC');
       
    % define longitudinal states for pitch control
    DifferentialState x_h;      % altitude
         
    % define longitudinal controls for pitch
    Control uC_gamma;  % flight path, pitch - AoA
    
    % Parameters 
    Parameter p_alpha;  % angle of attack
    Parameter p_Vt;     % total speed

    t0 =    acado.MexInput;       % current sim time   
    %T_s =  acado.MexInput;       % sampling time not accepted as mex input  
    T_h =   acado.MexInput;       % T_horizon
    N =     acado.MexInput;       % number of intervals in horizon
    xu0 =   acado.MexInputVector; % vector input necessary for sim init 
    xRef =  acado.MexInputVector;
    xRefT = acado.MexInputMatrix; 
    Q_R =   acado.MexInputMatrix; % Q & R matching the cost state variables
    p0 =    acado.MexInputVector; % parameter init values
    S =     acado.MexInputMatrix; % S, matching the terminal cost variables
    sRef =  acado.MexInputVector;
    p0_alpha  = acado.MexInput;      % thrust cmd parameter
    p0_Vt= acado.MexInput;      % qbar, dynamic pressure
  
    % current sim time for start of a new prediction horizon
    startime =  t0;        
    T_s = 0.5;     % T_sample (200Hz), acado doesn't accept mex input!
    t_start	= 0.0;  % start of horizon ????
    t_end	= T_h;  % end/length of horizon
    d2r = pi/180.0; % degrees to radians
    r2d = 180.0/pi; % radians to degrees
    
    % f16 operating range/limits (and model validity limits)     
    min_Vt = 300;           % [ft/s]
    max_Vt = 900;           % [ft/s]
    min_alpha = -10.0*d2r;  % [rad]
    max_alpha =  45.0*d2r;  % [rad]
    % control limits
    min_gamma = -25.0*d2r;  % [rad]
    max_gamma =  25.0*d2r;  % [rad]
    min_h_dot = -833.3;     % [ft/s]
    max_h_dot =  833.3;     % [ft/s]
    
    % intermediate variables
	U = p_Vt*cos(p_alpha);
    W = p_Vt*sin(p_alpha); 
    h_dot = U*sin(uC_gamma+p_alpha) - W*cos(uC_gamma+p_alpha);
        
    %% Differential Equation
    f = acado.DifferentialEquation();   % the differential equation object    
    f.add( dot(x_h) == h_dot ); % h_dot
    
    %% Optimal Control Problem
    ocp = acado.OCP(t_start, t_end, N );    % examine both t_start=t0 & t_end=t0+T_h    
    
    % the cost fn 
    % h={running cost} as a LSQ-Function 
    h={x_h, uC_gamma};
    
    % must match number of states and control variables in the cost func.
    r = xRef;
    
    ocp.minimizeLSQ(Q_R, h, r); 
    
    % constraints
    ocp.subjectTo( f );  % the aircraft altitude control model     
    
    ocp.subjectTo( min_gamma <= uC_gamma <= max_gamma ); 
    ocp.subjectTo( min_h_dot <= h_dot <= max_h_dot ); % x_h_dot eqn req
    
    % parameter constraints
    ocp.subjectTo( p0_alpha == p_alpha );    %
    ocp.subjectTo( p0_Vt == p_Vt );    %  


    %% SETTING UP THE MPC CONTROLLER:
    algo = acado.RealTimeAlgorithm(ocp, T_s);  
    
%     algo.set('MAX_NUM_ITERATIONS', 3 ); 
%     algo.set('DISCRETIZATION_TYPE', 'SINGLE_SHOOTING' );
%     algo.set('INTEGRATOR_TYPE', 'INT_BDF' );
    
    %% CONTROLLER
    
    % reference trajectory
    ref = xRefT;    
    reference = acado.StaticReferenceTrajectory(ref);    
                                                        
    controller = acado.Controller( algo,reference ); 
 
    controller.init(startime, xu0, p0);     
    controller.step(startime, xu0);


END_ACADO;    

