%--------------------------------------------------------------------------
% file/function name: fpath_NMPC_reconfig 
%--------------------------------------------------------------------------

% clear;
BEGIN_ACADO; 
    
    % Acado function name property
    acadoSet('problemname', 'fpath_NMPC_reconfig');
       
    % define longitudinal states for pitch control
    DifferentialState x_gamma;   % flight path, pitch - AoA
    DifferentialState x_Q;       % pitch rate, theta_dot
    DifferentialState x_alpha;   % angle of attack
    DifferentialState x_Vt;      % total speed
    DifferentialState u_el1;     % left stabilator actuator dynamics
    DifferentialState u_el1_dot; % left stabilator actuator rate control/limits
    DifferentialState u_el2;     % right stabilator actuator dynamics
    DifferentialState u_el2_dot; % right stabilator actuator rate control/limits
         
    % define longitudinal controls for pitch
    Control uC_el1;  % left stabilator/elevator deflection cmd 
    Control uC_el2;  % right stabilator/elevator deflection cmd 
    Control uC_ft;   % thrust force cmd parameter
    
    % Parameters
    Parameter p_qbar;% dynamic pressure parameter   

    t0 =    acado.MexInput;       % current sim time   
    %T_s =  acado.MexInput;       % sampling time not accepted as mex input  
    T_h =   acado.MexInput;       % T_horizon
    N =     acado.MexInput;       % number of intervals in horizon
    xu0 =   acado.MexInputVector; % vector input necessary for sim init 
    xRef =  acado.MexInputVector;
    xRefT = acado.MexInputMatrix; 
    Q_R =   acado.MexInputMatrix; % Q & R matching the cost state variables
    p0 =    acado.MexInputVector; % parameter init values
    S =     acado.MexInputMatrix; % S, matching the terminal cost variables
    sRef =  acado.MexInputVector;
    min_el1 = acado.MexInput;      % [deg]
    max_el1 = acado.MexInput;      % [deg]
    min_el2 = acado.MexInput;      % [deg]
    max_el2 = acado.MexInput;      % [deg]
    min_el1_dot = acado.MexInput;  % [deg/s]
    max_el1_dot = acado.MexInput;  % [deg/s]
    min_el2_dot = acado.MexInput;  % [deg/s]
    max_el2_dot = acado.MexInput;  % [deg/s]
    p0_ft  = acado.MexInput;       % thrust cmd parameter
    p0_qbar= acado.MexInput;       % qbar, dynamic pressure
    s_min_gamma = acado.MexInput;  % dynamic stabilty constraint limit
    s_max_gamma = acado.MexInput;  % dynamic stabilty constraint limit
    s_min_alpha = acado.MexInput;  % dynamic stabilty constraint limit
    s_max_alpha = acado.MexInput;  % dynamic stabilty constraint limit
  
    % current sim time for start of a new prediction horizon
    startime =  t0;        
    T_s = 0.05;     % T_sample (200Hz), acado doesn't accept mex input!
    t_start	= 0.0;  % start of horizon ????
    t_end	= T_h;  % end/length of horizon
    d2r = pi/180.0; % degrees to radians
    r2d = 180.0/pi; % radians to degrees
    
    % f16 operating range/limits (and model validity limits)
    min_Vt = 300;           % [f/s]
    max_Vt = 900;           % [f/s]
    min_ft = 1000.0;        % [lb]
    max_ft = 19000.0;       % [lb]
    min_alpha = -10.0*d2r;  % [rad]
    max_alpha = 45.0*d2r;   % [rad]
    % control limits
    min_gamma = -45.0*d2r;  % [rad]
    max_gamma = 45.0*d2r;   % [rad]
        
    %% Differential Equation
    f = acado.DifferentialEquation();   % the differential equation object    
    f.linkCFunction('aerodata_cfncPath_reconfig.c', 'aerodata'); % link c black-box
    
    %% Optimal Control Problem
    ocp = acado.OCP(t_start, t_end, N );    % examine both t_start=t0 & t_end=t0+T_h    
    
    % the cost fn 
    % h={running cost} as a LSQ-Function 
    h={x_gamma, x_Q, x_alpha, x_Vt, uC_el1, uC_el2, uC_ft};
    
    % m={terminal cost} as a LSQ-Function 
    m={x_gamma};
        
    % must match number of states and control variables in the cost func.
    r = xRef;
    rs= sRef;
    
    ocp.minimizeLSQ(Q_R, h, r); 
%     ocp.minimizeLSQEndTerm(S, m, rs);
    
    % constraints
    ocp.subjectTo( f );  % the aircraft pitch control model                   
    ocp.subjectTo( min_el1 <= uC_el1 <= max_el1 );  % el deflection cmd limits
    ocp.subjectTo( min_el1_dot <= u_el1_dot <= max_el1_dot ); % el rate limits  
    ocp.subjectTo( min_el2 <= uC_el2 <= max_el2 );  % el deflection cmd limits
    ocp.subjectTo( min_el2_dot <= u_el2_dot <= max_el2_dot ); % el rate limits  
    
    % option: include longitudinal stab. speeds and thrust control
    ocp.subjectTo( min_alpha <= x_alpha <= max_alpha ); 
    ocp.subjectTo( min_Vt <= x_Vt <= max_Vt );
    
    ocp.subjectTo( min_gamma <= x_gamma <= max_gamma ); 
    
    % parameter constraints
    ocp.subjectTo( min_ft <= uC_ft <= max_ft );% thrust dynamics not included     
    ocp.subjectTo( p0_qbar == p_qbar ); % dynamic pressure, same as initial  
    
    % stability or terminal constraints
    %ocp.subjectTo( 'AT_END', s_min_gamma <= x_theta <= s_max_gamma );   
    %ocp.subjectTo( 'AT_END', s_min_alpha <= x_alpha <= s_max_alpha ); 
    %ocp.subjectTo( 'AT_END', s_min_gamma <= x_theta <= s_max_gamma );   
    

    %% SETTING UP THE MPC CONTROLLER:
    algo = acado.RealTimeAlgorithm(ocp, T_s);  
    
%     algo.set('MAX_NUM_ITERATIONS', 3 ); 
%     algo.set('DISCRETIZATION_TYPE', 'SINGLE_SHOOTING' );
%     algo.set('INTEGRATOR_TYPE', 'INT_BDF' );
    
    %% CONTROLLER
    
    % reference trajectory
    ref = xRefT;    
    reference = acado.StaticReferenceTrajectory(ref);    
                                                        
    controller = acado.Controller( algo,reference ); 
 
    controller.init(startime, xu0, p0);     
    controller.step(startime, xu0);


END_ACADO;    

