%--------------------------------------------------------------------------
% file/function name: heading_NMPC 
%--------------------------------------------------------------------------

% clear;
BEGIN_ACADO; 
    
    % Acado function name property
    acadoSet('problemname', 'heading_NMPC');
       
    % define lateral states for pitch control
    DifferentialState x_psi;            % heading (yaw)
         
    % define lateral controls
    Control uC_phi;  % roll cmd 
    
    % Parameters
    Parameter p_Vt;   % speed  
    
    % Disturbance
    Disturbance d_beta;     % drift in heading (sideslip)
    
    t0 =    acado.MexInput;       % current sim time   
    %T_s =  acado.MexInput;       % sampling time not accepted as mex input  
    T_h =   acado.MexInput;       % T_horizon
    N =     acado.MexInput;       % number of intervals in horizon
    xu0 =   acado.MexInputVector; % vector input necessary for sim init 
    xRef =  acado.MexInputVector;
    xRefT = acado.MexInputMatrix; 
    Q_R =   acado.MexInputMatrix; % Q & R matching the cost state variables
    p0 =    acado.MexInputVector; % parameter init values
    S =     acado.MexInputMatrix; % S, matching the terminal cost variables
    sRef =  acado.MexInputVector;
    p0_Vt = acado.MexInput;      % speed
    d0_beta= acado.MexInput;
    
    % current sim time for start of a new prediction horizon
    startime =  t0;        
    T_s = 0.5;     % T_sample (200Hz), acado doesn't accept mex input!
    t_start	= 0.0;  % start of horizon
    t_end	= T_h;  % end/length of horizon
    d2r = pi/180.0; % degrees to radians
    r2d = 180.0/pi; % radians to degrees
    
    % control and state limits
    min_phi = -65.0*d2r;    % -65.0 tested [rad]
    max_phi =  65.0*d2r;    %  65.0 [rad] 
    
    % gravity constant
    g = 32.17;   % [ft/s^2]
    
    % intermediate staes
    hd= x_psi - 0.0*d_beta; % 0.25 tested well!
    
    x_R = g*sin(uC_phi)/p_Vt;
    
    %% Differential Equation
    f = acado.DifferentialEquation();   % the differential equation object  
    f.add(dot(x_psi) == x_R ); % psi_dot
    
    %% Optimal Control Problem
    ocp = acado.OCP(t_start, t_end, N );    % examine both t_start=t0 & t_end=t0+T_h    
    
    % the cost fn ;
    % h={running cost} as a LSQ-Function 
    h={hd, uC_phi};
    
    % m={terminal cost} as a LSQ-Function 
%     m={x_psi};
    
    % must match number of states and control variables in the cost func.
    r = xRef;
    rs= sRef;
    
    ocp.minimizeLSQ(Q_R, h, r); 
%     ocp.minimizeLSQEndTerm(S, m, rs);
    
    % constraints
    ocp.subjectTo( f );  % the aircraft pitch control model       
    ocp.subjectTo( min_phi <= uC_phi <= max_phi );
     
    % parameter constraints
    ocp.subjectTo( p0_Vt == p_Vt );     % constant dynamic pressure   
    ocp.subjectTo( d0_beta == d_beta );     % drift   

    %% SETTING UP THE MPC CONTROLLER:
    algo = acado.RealTimeAlgorithm(ocp, T_s);  
    
%     algo.set('MAX_NUM_ITERATIONS', 3 ); 
%     algo.set('DISCRETIZATION_TYPE', 'SINGLE_SHOOTING' );
%     algo.set('INTEGRATOR_TYPE', 'INT_BDF' );
    
    %% CONTROLLER
    
    % reference trajectory
    ref = xRefT;    
    reference = acado.StaticReferenceTrajectory(ref);    
                                                        
    controller = acado.Controller( algo,reference ); 
 
    controller.init(startime, xu0, p0);     
    controller.step(startime, xu0);


END_ACADO;    

