%--------------------------------------------------------------------------
% file/function name: pitch_NMPC 
%
% This file (lat_NMPC.m) is compiled by using the matlab run button (F5). 
% After compilation the file lat_NMPC_RUN can be run multiple times with 
% different parameters without needing to recompile again.
%  
% REMARK: the sequence of the parameters must be the same sequence as
%         how they were defined:
%
% run: 
% lat_NMPC_RUN(x0_alt, x0_theta, x0_Vt, x0_alpha, x0_Q, u0_el, u0_ft,
%                xu0{v}, t0, Q{m}, xRef{v}, xRefT{m}, Mt0, Xt0, Zt0, qbar0)
%
%--------------------------------------------------------------------------

% clear;
BEGIN_ACADO; 
    
    % Acado function name property
    acadoSet('problemname', 'lat_NMPC');
       
    % define lateral states for pitch control
    DifferentialState x_beta;   % sideslip angle
    DifferentialState x_psi;    % heading (yaw)
    DifferentialState x_phi;    % roll
    DifferentialState x_P;      % roll rate, theta_dot
    DifferentialState x_R;      % yaw rate, theta_dot
    DifferentialState u_ail;     % introduces actuator dynamics
    DifferentialState u_ail_dot; % introduces actuator rate control/limits
    DifferentialState u_rud;     % introduces actuator dynamics
    DifferentialState u_rud_dot; % introduces actuator rate control/limits
         
    % define lateral controls
    Control uC_ail;  % aileron deflection cmd 
    Control uC_rud;  % rudder deflection cmd
    
    % Parameters
    Parameter p_qbar;   % dynamic pressure   
    Parameter p_alpha;  % angle of attack (constant over T_h) 
    Parameter p_Vt;     % total speed parameter (constant over T_h)
    Parameter p_theta;  % theta parameter (constant over T_h) 
    Parameter p_ft;     % thrust force (constant over T_h)

    t0 =    acado.MexInput;       % current sim time   
    %T_s =  acado.MexInput;       % sampling time not accepted as mex input  
    T_h =   acado.MexInput;       % T_horizon
    N =     acado.MexInput;       % number of intervals in horizon
    xu0 =   acado.MexInputVector; % vector input necessary for sim init 
    xRef =  acado.MexInputVector;
    xRefT = acado.MexInputMatrix; 
    Q_R =   acado.MexInputMatrix; % Q & R matching the cost state variables
    p0 =    acado.MexInputVector; % parameter init values
    S =     acado.MexInputMatrix; % S, matching the terminal cost variables
    sRef =  acado.MexInputVector;
    min_ail = acado.MexInput;      % [deg]
    max_ail = acado.MexInput;      % [deg]
    min_rud = acado.MexInput;      % [deg]
    max_rud = acado.MexInput;      % [deg]
    p0_qbar = acado.MexInput;      % dynamic pressure parameter
    p0_alpha= acado.MexInput;      % alpha parameter
    p0_Vt   = acado.MexInput;      % Vt parameter
    p0_theta= acado.MexInput;      % theta parameter
    p0_ft   = acado.MexInput;      % thrust force parameter

    % current sim time for start of a new prediction horizon
    startime =  t0;        
    T_s = 0.05;     % T_sample (200Hz), acado doesn't accept mex input!
    t_start	= 0.0;  % start of horizon ????
    t_end	= T_h;  % end/length of horizon
    d2r = pi/180.0; % degrees to radians
    r2d = 180.0/pi; % radians to degrees
    
    % f16 operating range/limits (and model validity limits)
    min_ail_dot = -80.0;    % [deg/s]
    max_ail_dot =  80.0;    % [deg/s]     
    min_rud_dot = -120.0;   % [deg/s]
    max_rud_dot =  120.0;   % [deg/s]     
    min_Vt = 300;           % [f/s]
    map_Vt = 900;           % [f/s]
    min_ft = 1000.0;        % [lb]
    max_ft = 19000.0;       % [lb]
    min_alpha = -10.0*d2r;  % [rad]
    max_alpha =  45.0*d2r;  % [rad]
    % control limits
    min_phi = -65.0*d2r;    % [rad]
    max_phi =  65.0*d2r;    % [rad]
    min_P   = -308.0*d2r;    % [rad/s]
    max_P   =  308.0*d2r;    % [rad/s]
    
        
    %% Differential Equation
    f = acado.DifferentialEquation();   % the differential equation object    
    f.linkCFunction('aerodata_cfncLa.c', 'aerodata'); % link c black-box
    
    %% Optimal Control Problem
    ocp = acado.OCP(t_start, t_end, N );    % examine both t_start=t0 & t_end=t0+T_h    
    
    % the cost fn variables {x_theta, p_Vt, p_alpha, x_P, u_ail, u_ft}?;
    % h={running cost} as a LSQ-Function 
%     h={x_beta, x_phi, x_psi, x_R, uC_ail, uC_rud};
    h={x_beta, x_phi, x_psi, x_P, x_R, uC_ail, uC_rud};
    
    % m={terminal cost} as a LSQ-Function 
    m={x_beta, x_phi, x_psi, x_P, x_R};
    
    % r: reference comes in as a mex input vector
    % ref = [ r_theta   r_Q   r_alpha   r_Vt   0  ft_trim ];
    % ref = [ r_theta   0 ];
    
    % must match number of states and control variables in the cost func.
    r = xRef;
    rs= sRef;
    
    ocp.minimizeLSQ(Q_R, h, r); 
%     ocp.minimizeLSQEndTerm(S, m, rs);
    
    % constraints
    ocp.subjectTo( f );  % the aircraft pitch control model                   
    ocp.subjectTo( min_ail <= uC_ail <= max_ail );  % ail deflection cmd limits
    ocp.subjectTo( min_ail_dot <= u_ail_dot <= max_ail_dot ); % ail rate limits  
    ocp.subjectTo( min_rud <= uC_rud <= max_rud );  % rud deflection cmd limits
    ocp.subjectTo( min_rud_dot <= u_rud_dot <= max_rud_dot ); % rud rate limits  
    
    ocp.subjectTo( min_phi <= x_phi <= max_phi );
    ocp.subjectTo( min_P <= x_P <= max_P );
    
    % parameter constraints
    ocp.subjectTo( p0_qbar == p_qbar );     % constant dynamic pressure
    ocp.subjectTo( p0_alpha == p_alpha );   % constant AoA 
    ocp.subjectTo( p0_Vt == p_Vt );         % constant speed 
    ocp.subjectTo( p0_theta == p_theta );   % constant pitch 
    ocp.subjectTo( p0_ft == p_ft );         % constant thrust 
%     ocp.subjectTo( 'AT_START', p0_qbar == p_qbar );     % constant dynamic pressure
%     ocp.subjectTo( 'AT_END', p0_qbar == p_qbar );     % constant dynamic pressure
%     ocp.subjectTo( 'AT_START', p0_alpha == p_alpha );   % constant AoA 
%     ocp.subjectTo( 'AT_END', p0_alpha == p_alpha );   % constant AoA 
%     ocp.subjectTo( 'AT_START', p0_Vt == p_Vt );         % constant speed 
%     ocp.subjectTo( 'AT_END', p0_Vt == p_Vt );         % constant speed 
%     ocp.subjectTo( 'AT_START', p0_theta == p_theta );   % constant pitch 
%     ocp.subjectTo( 'AT_END', p0_theta == p_theta );   % constant pitch 
%     ocp.subjectTo( 'AT_START', p0_ft == p_ft );         % constant thrust 
%     ocp.subjectTo( 'AT_END', p0_ft == p_ft );         % constant thrust 

%     ocp.subjectTo( 'AT_START', p0_qbar == p_qbar );    % or use same as initial value :(  
%     ocp.subjectTo( 'AT_END', p0_qbar == p_qbar );    % or use same as initial value :(      
    

    %% SETTING UP THE MPC CONTROLLER:
    algo = acado.RealTimeAlgorithm(ocp, T_s);  
    
%     algo.set('MAX_NUM_ITERATIONS', 3 ); 
%     algo.set('DISCRETIZATION_TYPE', 'SINGLE_SHOOTING' );
%     algo.set('INTEGRATOR_TYPE', 'INT_BDF' );
    
    %% CONTROLLER
    
    % reference trajectory
    ref = xRefT;    
    reference = acado.StaticReferenceTrajectory(ref);    
                                                        
    controller = acado.Controller( algo,reference ); 
 
    controller.init(startime, xu0, p0);     
    controller.step(startime, xu0);


END_ACADO;    

