%--------------------------------------------------------------------------
% file/function name: pitch_NMPC 
%
% This file (pitch_NMPC.m) is compiled by using the matlab run button (F5). 
% After compilation the file pitch_NMPC_RUN can be run multiple times with 
% different parameters without needing to recompile again.
%  
% REMARK: the sequence of the parameters must be the same sequence as
%         how they were defined:
%
% run: 
% pitch_NMPC_RUN(x0_alt, x0_theta, x0_Vt, x0_alpha, x0_Q, u0_el, u0_ft,
%                xu0{v}, t0, Q{m}, xRef{v}, xRefT{m}, Mt0, Xt0, Zt0, qbar0)
%
%--------------------------------------------------------------------------

% clear;
BEGIN_ACADO; 
    
    % Acado function name property
    acadoSet('problemname', 'long_NMPC');
       
    % define longitudinal states for pitch control
    DifferentialState x_h;      % altitude
    DifferentialState x_gamma;  % flight path, pitch - AoA
    DifferentialState x_Q;      % pitch rate, theta_dot
    DifferentialState x_alpha;  % angle of attack
    DifferentialState x_Vt;     % total speed
    DifferentialState u_el;     % introduces actuator dynamics
    DifferentialState u_el_dot; % introduces actuator rate control/limits
         
    % define longitudinal controls for pitch
    Control uC_el;  % elevator deflection cmd 
    Control uC_ft;  % thrust force cmd parameter
    
    % Parameters
    Parameter p_qbar;% dynamic pressure parameter   

    t0 =    acado.MexInput;       % current sim time   
    %T_s =  acado.MexInput;       % sampling time not accepted as mex input  
    T_h =   acado.MexInput;       % T_horizon
    N =     acado.MexInput;       % number of intervals in horizon
    xu0 =   acado.MexInputVector; % vector input necessary for sim init 
    xRef =  acado.MexInputVector;
    xRefT = acado.MexInputMatrix; 
    Q_R =   acado.MexInputMatrix; % Q & R matching the cost state variables
    p0 =    acado.MexInputVector; % parameter init values
    S =     acado.MexInputMatrix; % S, matching the terminal cost variables
    sRef =  acado.MexInputVector;
    min_el = acado.MexInput;      % [deg]
    max_el = acado.MexInput;      % [deg]
    p0_ft  = acado.MexInput;      % thrust cmd parameter
    p0_qbar= acado.MexInput;      % qbar, dynamic pressure
    s_min_theta = acado.MexInput; % dynamic stabilty constraint limit
    s_max_theta = acado.MexInput; % dynamic stabilty constraint limit
    s_min_alpha = acado.MexInput; % dynamic stabilty constraint limit
    s_max_alpha = acado.MexInput; % dynamic stabilty constraint limit
  
    % current sim time for start of a new prediction horizon
    startime =  t0;        
    T_s = 0.05;     % T_sample (200Hz), acado doesn't accept mex input!
    t_start	= 0.0;  % start of horizon ????
    t_end	= T_h;  % end/length of horizon
    d2r = pi/180.0; % degrees to radians
    r2d = 180.0/pi; % radians to degrees
    
    % f16 operating range/limits (and model validity limits)
    min_el_dot = -60.0;     % [deg/s]
    max_el_dot = 60.0;      % [deg/s]     
    min_Vt = 300;           % [f/s]
    max_Vt = 900;           % [f/s]
    min_ft = 1000.0;        % [lb]
    max_ft = 19000.0;       % [lb]
    min_alpha = -10.0*d2r;  % [rad]
    max_alpha = 45.0*d2r;   % [rad]
    % control limits
    min_gamma = -45.0*d2r;  % [rad]
    max_gamma = 45.0*d2r;   % [rad]
    min_h_dot = -98.425;    % [f/s] 30m/s
    max_h_dot = 98.425;     % [f/s]
        
    %% Differential Equation
    f = acado.DifferentialEquation();   % the differential equation object    
    f.linkCFunction('aerodata_cfncL.c', 'aerodata'); % link c black-box
    
    %% Optimal Control Problem
    ocp = acado.OCP(t_start, t_end, N );    % examine both t_start=t0 & t_end=t0+T_h    
    
    % the cost fn variables {x_theta, x_Vt, x_alpha, x_Q, u_el, u_ft}?;
    % h={running cost} as a LSQ-Function 
%     h={x_theta, u_el};
%     h={x_theta, x_alpha, u_el, u_ft, qbar};
%     h={x_theta, x_alpha, uC_el, uC_ft, x_Vt, qbar};
%     h={x_theta, x_alpha, uC_el, uC_ft, qbar};
    h={x_h, x_gamma, x_Q, x_alpha, uC_el, uC_ft};
    
    % m={terminal cost} as a LSQ-Function 
%     m={x_theta, x_alpha};
    
    % r: reference comes in as a mex input vector
    % ref = [ r_theta   r_Q   r_alpha   r_Vt   0  ft_trim ];
    % ref = [ r_theta   0 ];
    
    % must match number of states and control variables in the cost func.
    r = xRef;
    rs= sRef;
    
    ocp.minimizeLSQ(Q_R, h, r); 
%     ocp.minimizeLSQEndTerm(S, m, rs);
    
    % constraints
    ocp.subjectTo( f );  % the aircraft pitch control model                   
    ocp.subjectTo( min_el <= uC_el <= max_el );  % el deflection cmd limits
    ocp.subjectTo( min_el_dot <= u_el_dot <= max_el_dot ); % el rate limits  
    
    % option: include longitudinal stab. speeds and thrust control
    ocp.subjectTo( min_alpha <= x_alpha <= max_alpha ); 
    ocp.subjectTo( min_Vt <= x_Vt <= max_Vt );
    
    ocp.subjectTo( min_gamma <= x_gamma <= max_gamma ); 
    % ocp.subjectTo( min_h_dot <= x_h_dot <= max_h_dot ); % x_h_dot eqn req
    
    % parameter constraints
    ocp.subjectTo( min_ft <= uC_ft <= max_ft );% thrust dynamics not included
%     ocp.subjectTo( 'AT_START', p0_ft == p_ft);    % or use same as initial value :(  
%     ocp.subjectTo( 'AT_END', p0_ft == p_ft);    % or use same as initial value :(  
    ocp.subjectTo( 'AT_START', p0_qbar == p_qbar );    % or use same as initial value :(  
    ocp.subjectTo( 'AT_END', p0_qbar == p_qbar );    % or use same as initial value :(      
%     ocp.subjectTo( p0_qbar == p_qbar ); % dynamic pressure, same as initial  
    
    % stability or terminal constraints
    %ocp.subjectTo( 'AT_END', s_min_theta <= x_theta <= s_max_theta );   
%     ocp.subjectTo( 'AT_END', s_min_alpha <= x_alpha <= s_max_alpha ); 


    %% SETTING UP THE MPC CONTROLLER:
    algo = acado.RealTimeAlgorithm(ocp, T_s);  
    
%     algo.set('MAX_NUM_ITERATIONS', 3 ); 
%     algo.set('DISCRETIZATION_TYPE', 'SINGLE_SHOOTING' );
%     algo.set('INTEGRATOR_TYPE', 'INT_BDF' );
    
    %% CONTROLLER
    
    % reference trajectory
    ref = xRefT;    
    reference = acado.StaticReferenceTrajectory(ref);    
                                                        
    controller = acado.Controller( algo,reference ); 
 
    controller.init(startime, xu0, p0);     
    controller.step(startime, xu0);


END_ACADO;    

