%--------------------------------------------------------------------------
% file/function name: pitch_NMPC 
%
% This file (pitch_NMPC.m) is compiled by using the matlab run button (F5). 
% After compilation the file pitch_NMPC_RUN can be run multiple times with 
% different parameters without needing to recompile again.
%  
% REMARK: the sequence of the parameters must be the same sequence as
%         how they were defined:
%
% run: 
% pitch_NMPC_RUN(x0_alt, x0_theta, x0_Vt, x0_alpha, x0_Q, u0_el, u0_ft,
%                xu0{v}, t0, Q{m}, xRef{v}, xRefT{m}, Mt0, Xt0, Zt0, qbar0)
%
%--------------------------------------------------------------------------

% clear;

BEGIN_ACADO; 
    
    % Acado function name property
    acadoSet('problemname', 'pitch_NMPC');
       
    % define longitudinal states for pitch control
    DifferentialState x_theta;  
    DifferentialState x_Q;
    DifferentialState x_alpha;
    DifferentialState x_Vt;
         
    % define longitudinal controls for pitch
    Control u_el;   % elevator deflection 
    Control u_ft;   % thrust force
    
    % define longitudinal aerodynamic parameters
    Parameter Czq Cz dZdQ Cz_tot; 
    Parameter Cmq Cm dMdQ Cm_tot;
    Parameter Cxq Cx dXdQ Cx_tot;
    Parameter M_tot X_tot Z_tot; 
    Parameter D L;

    t0 =    acado.MexInput;       % current sim time   
    %T_s =   acado.MexInput;      % sampling time not accepted as mex input  
    T_h =   acado.MexInput;       % T_horizon
    N =     acado.MexInput;       % number of intervals in horizon
    xu0 =   acado.MexInputVector; % vector input necessary for sim init 
    xRef =  acado.MexInputVector;
    xRefT = acado.MexInputMatrix; 
    Q =     acado.MexInputMatrix; % matching the cost state variables
    
  
    % current sim time for start of a new prediction horizon
    startime =  t0;        
    T_s = 0.005;     % T_sample, acado doesn't accept mex input!
    t_start	= 0.0;  % start of horizon ????
    t_end	= T_h;  % end/length of horizon
    
    % f16 operating range
    min_el = -15.0;
    max_el = 15.0;
    min_Vt = 300;
    max_Vt = 900;
    min_ft = 1000.0;
    max_ft = 19000.0;
    min_alpha = -10.0;
    max_alpha = 45.0;

    %% Differential Equation
    f = acado.DifferentialEquation();   % the differential equation object
    
    f.linkCFunction('aerodata_cfncL.c', 'aerodata');
    
    %% Optimal Control Problem
    ocp = acado.OCP(t_start, t_end, N );    
    
    % the cost fn variables {x_theta, x_Vt, x_alpha, x_Q, u_el, u_ft};

    % h={cost} the LSQ-Function 
    h={x_theta, x_Q, x_alpha, x_Vt, u_el, u_ft};
    
    % r: reference comes in as a mex input vector
    % ref = [ r_theta   r_Q   r_alpha   r_Vt   0  ft_trim ];
    
    % must match number of stetes and control variables in the cost func.
    r = xRef;
    
    ocp.minimizeLSQ(Q, h, r); 
    
    ocp.subjectTo( f );                     
    ocp.subjectTo( min_el <= u_el <= max_el ); 
    
    % option: include longitudinal stab. speeds in the controls if required
    ocp.subjectTo( min_Vt <= x_Vt <= max_Vt );
    ocp.subjectTo( min_ft <= u_ft <= max_ft ); 
    ocp.subjectTo( min_alpha <= x_alpha <= max_alpha );
        
    
    %% SETTING UP THE MPC CONTROLLER:
    algo = acado.RealTimeAlgorithm(ocp, T_s);  
    
    % algo.set('MAX_NUM_ITERATIONS', 3 ); 
    % algo.set('HESSIAN_APPROXIMATION', 'GAUSS_NEWTON' );
    
    
    %% CONTROLLER
    
    % reference trajectory
    % options: line, spline etc to link initial and final ref values
    % ref = [ startime      x0_theta  x0_Q  x0_alpha  x0_Vt  0  ft_trim
    %         startime+T_h  r_theta   r_Q   r_alpha   r_Vt   0  ft_trim ];        
     
    ref = xRefT;

    reference = acado.StaticReferenceTrajectory(ref);    
                                                        
    controller = acado.Controller( algo,reference ); 
 
    controller.init(startime, xu0);     
    controller.step(startime, xu0);


END_ACADO;    

