%--------------------------------------------------------------------------
% file/function name: roll_NMPC_reconfig 
%--------------------------------------------------------------------------

% clear;
BEGIN_ACADO; 
    
    % Acado function name property
    acadoSet('problemname', 'roll_NMPC_reconfig');
       
    % define lateral states for pitch control
    DifferentialState x_beta;   % sideslip angle
%     DifferentialState x_psi;    % heading (yaw)
    DifferentialState x_phi;    % roll
    DifferentialState x_P;      % roll rate, theta_dot
    DifferentialState x_R;      % yaw rate, theta_dot
    DifferentialState u_ail1;     % left aileron actuator state
    DifferentialState u_ail1_dot; % left aileron actuator rate control/limits
    DifferentialState u_ail2;     % right aileron actuator state
    DifferentialState u_ail2_dot; % right aileron rate control/limits
    DifferentialState u_rud;     % introduces actuator dynamics
    DifferentialState u_rud_dot; % introduces actuator rate control/limits
         
    % define lateral controls
    Control uC_ail1;    % left aileron deflection cmd 
    Control uC_ail2;    % right aileron deflection cmd 
    Control uC_rud;     % rudder deflection cmd
    
    % Parameters
    Parameter p_qbar;   % dynamic pressure   
    Parameter p_alpha;  % angle of attack (constant over T_h) 
    Parameter p_Vt;     % total speed parameter (constant over T_h)
    Parameter p_theta;  % theta parameter (constant over T_h) 
    Parameter p_ft;     % thrust force (constant over T_h)
    Parameter p_X_tot;  % total force in X 
    Parameter p_Z_tot;  % total force in Z           

    t0 =    acado.MexInput;       % current sim time   
    %T_s =  acado.MexInput;       % sampling time not accepted as mex input  
    T_h =   acado.MexInput;       % T_horizon
    N =     acado.MexInput;       % number of intervals in horizon
    xu0 =   acado.MexInputVector; % vector input necessary for sim init 
    xRef =  acado.MexInputVector;
    xRefT = acado.MexInputMatrix; 
    Q_R =   acado.MexInputMatrix; % Q & R matching the cost state variables
    p0 =    acado.MexInputVector; % parameter init values
    S =     acado.MexInputMatrix; % S, matching the terminal cost variables
    sRef =  acado.MexInputVector;
    min_ail1= acado.MexInput;     % [deg]
    max_ail1= acado.MexInput;     % [deg]
    min_ail2= acado.MexInput;     % [deg]
    max_ail2= acado.MexInput;     % [deg]
    min_rud = acado.MexInput;     % [deg]
    max_rud = acado.MexInput;     % [deg]
    min_ail1_dot= acado.MexInput; % [deg/s]
    max_ail1_dot= acado.MexInput; % [deg/s]
    min_ail2_dot= acado.MexInput; % [deg/s]
    max_ail2_dot= acado.MexInput; % [deg/s]
    min_rud_dot = acado.MexInput; % [deg/s]
    max_rud_dot = acado.MexInput; % [deg/s]
    p0_qbar = acado.MexInput;     % dynamic pressure parameter
    p0_alpha= acado.MexInput;     % alpha parameter
    p0_Vt   = acado.MexInput;     % Vt parameter
    p0_theta= acado.MexInput;     % theta parameter
    p0_ft   = acado.MexInput;     % thrust force parameter
    p0_X_tot= acado.MexInput;     % total force in X parameter
    p0_Z_tot= acado.MexInput;     % total force in Z parameter

    % current sim time for start of a new prediction horizon
    startime =  t0;        
    T_s = 0.05;     % T_sample (200Hz), acado doesn't accept mex input!
    t_start	= 0.0;  % start of horizon
    t_end	= T_h;  % end/length of horizon
    d2r = pi/180.0; % degrees to radians
    r2d = 180.0/pi; % radians to degrees
    
    % f16 operating range/limits (and model validity limits)
    min_beta  = -30.0*d2r;  % [rad]
    max_beta  =  30.0*d2r;  % [rad]
    % control limits
    min_P   = -308.0*d2r;    % [rad/s]
    max_P   =  308.0*d2r;    % [rad/s]
    
        
    %% Differential Equation
    f = acado.DifferentialEquation();   % the differential equation object    
    f.linkCFunction('aerodata_cfncRoll_reconfig.c', 'aerodata'); % link c black-box
    
    %% Optimal Control Problem
    ocp = acado.OCP(t_start, t_end, N );    % examine both t_start=t0 & t_end=t0+T_h    
    
    % the cost fn variables;
    % h={running cost} as a LSQ-Function 
    h={x_beta, x_phi, x_P, x_R, uC_ail1, uC_ail2, uC_rud};
    
    % m={terminal cost} as a LSQ-Function 
    m={x_beta, x_phi, x_psi, x_P, x_R};
    
    % must match number of states and control variables in the cost func.
    r = xRef;
    rs= sRef;
    
    ocp.minimizeLSQ(Q_R, h, r); 
%     ocp.minimizeLSQEndTerm(S, m, rs);
    
    % constraints
    ocp.subjectTo( f );  % the aircraft pitch control model                   
    ocp.subjectTo( min_ail1 <= uC_ail1 <= max_ail1 );  
    ocp.subjectTo( min_ail2 <= uC_ail2 <= max_ail2 ); 
    ocp.subjectTo( min_ail1_dot <= u_ail1_dot <= max_ail1_dot );   
    ocp.subjectTo( min_ail2_dot <= u_ail2_dot <= max_ail2_dot );  
    ocp.subjectTo( min_rud <= uC_rud <= max_rud );  
    ocp.subjectTo( min_rud_dot <= u_rud_dot <= max_rud_dot );   
    
%     ocp.subjectTo( min_phi <= x_phi <= max_phi );
    ocp.subjectTo( min_beta <= x_beta <= max_beta );
    ocp.subjectTo( min_P <= x_P <= max_P );
%     ocp.subjectTo( min_R <= x_R <= max_R );
    
    % parameter constraints
    ocp.subjectTo( p0_qbar == p_qbar );    
    ocp.subjectTo( p0_alpha == p_alpha );   
    ocp.subjectTo( p0_Vt == p_Vt );         
    ocp.subjectTo( p0_theta == p_theta );   
    ocp.subjectTo( p0_ft == p_ft );         
    ocp.subjectTo( p0_X_tot == p_X_tot );       
    ocp.subjectTo( p0_Z_tot == p_Z_tot );         
    

    %% SETTING UP THE MPC CONTROLLER:
    algo = acado.RealTimeAlgorithm(ocp, T_s);  
    
%     algo.set('MAX_NUM_ITERATIONS', 3 ); 
%     algo.set('DISCRETIZATION_TYPE', 'SINGLE_SHOOTING' );
%     algo.set('INTEGRATOR_TYPE', 'INT_BDF' );
    
    %% CONTROLLER
    
    % reference trajectory
    ref = xRefT;    
    reference = acado.StaticReferenceTrajectory(ref);    
                                                        
    controller = acado.Controller( algo,reference ); 
 
    controller.init(startime, xu0, p0);     
    controller.step(startime, xu0);


END_ACADO;    

