function fnc_out = roll_autopilot_reconfig(lat_states, phi_ref, t0, p0_qbar, u0_ail1, u0_ail2, u0_rud, p0_ft, long_forces)
%----------------------------------------------------------
% A wrapper for lat_NMPC function
%----------------------------------------------------------

% extract states and parameters from func. input
x0_phi    = lat_states(1);
p0_theta  = lat_states(2);
% x0_psi    = lat_states(3);
p0_Vt     = lat_states(4);
p0_alpha  = lat_states(5);
x0_beta   = lat_states(6);
x0_P      = lat_states(7);
x0_R      = lat_states(8);

p0_X_tot = long_forces(2); 
p0_Z_tot = long_forces(3);

% initialize with current state value for continuity
u0_ail1_dot=0.0;
u0_ail2_dot=0.0;
u0_rud_dot=0.0;

xu0 = [x0_beta, x0_phi, x0_P, x0_R, u0_ail1, u0_ail1_dot, u0_ail2, u0_ail2_dot, u0_rud, u0_rud_dot];
p0 =  [p0_qbar, p0_alpha, p0_Vt, p0_theta, p0_ft, p0_X_tot, p0_Z_tot]; % for parameter init

% optimal control parameters
% prediction horizon 
% (trade off between optimality and online computational load)

T_s = 0.05;     % NB!!! sampling time not accepted as acado mex input
T_h = 1.0;      % pridiction and control horizon
N = T_h/T_s;    

% reference trajectory
d2r = pi/180.0; % degrees to radians
r_phi = phi_ref*d2r;% roll reference [rad] 
r_beta = 0.0;       % sideslip ref [rad]
r_P = 0.0; % roll rate reference [rad/s]
r_R = 0.0; % yaw rate reference [rad/s] 
r_rud = u0_rud;      % 0.0 good for optimal trajectory gen, u0_rud best for fault tolerance rudder ref
r_ail1 = u0_ail1;      % aileron ref
r_ail2 = u0_ail2;      % aileron ref

% ref trajectory option 1: straight line link, t0 available in workspace       
% optimization reference
ref = [ r_beta  r_phi  r_P  r_R  r_ail1  r_ail2  r_rud ];    % running cost reference
sRef= [ r_phi r_P  r_R ];    % terminal cost reference

% reference trajectory generation
tS1 = t0+(T_h/5.0); % 1st coincidence or intermediate time point (2*T_s?)
tS2 = t0+(T_h/2.0); % 2nd coincidence or intermediate time point (5*T_s?)

xS1_beta = x0_beta+(r_beta-x0_beta)*0.632; % assuming tS= time constant
xS1_phi = x0_phi+(r_phi-x0_phi)*0.632; % 

xS2_beta = x0_beta+(r_beta-x0_beta)*0.95; %
xS2_phi = x0_phi+(r_phi-x0_phi)*0.95; % 

% coincidence or intermediate points within the prediction horizon
time_ref0 = [t0 tS1 tS2 t0+T_h];

beta_ref0   = [x0_beta xS1_beta xS2_beta r_beta];
phi_ref0    = [p0_alpha xS1_phi xS2_phi r_phi];   

% ref trajectory generated using pchip 
% (Piecewise Cubic Hermite Interpolating Polynomial)
time_ref = t0: T_h/N : (t0+T_h); 
beta_ref = pchip(time_ref0, beta_ref0, time_ref);
phi_ref  = pchip(time_ref0, phi_ref0, time_ref);

% other constant refs 
rud_refC   = r_rud*ones(1,T_h/T_s+1); 
ail1_refC   = r_ail1*ones(1,T_h/T_s+1);
ail2_refC   = r_ail2*ones(1,T_h/T_s+1);
P_ref   = r_P*ones(1,T_h/T_s+1);
R_ref   = r_R*ones(1,T_h/T_s+1);

% the reference trajectory matrix for acado mex interface 
refT = [time_ref', beta_ref', phi_ref', P_ref', R_ref', ail1_refC', ail2_refC', rud_refC'];

% weighting matrices 
Q_R = eye(7);          % Q and R weight matrices
Q_R(1,1)= 1.0;         % beta, sideslip
Q_R(2,2)= 10.0;        % phi, roll
Q_R(3,3)= 1.0;         % P, roll rate
Q_R(4,4)= 1.0;         % R, yaw rate
Q_R(5,5)= 0.05;        % uC_ail1, aileron deflection cmd
Q_R(6,6)= 0.05;        % uC_ail2, aileron deflection cmd
Q_R(7,7)= 0.1;         % uC_rud,rudder cmd

% % terminal weight
% S = eye(2);      % P weight matrix
% S(1,1)= 1000.0;  % theta
% S(2,2)= 100.0;   % alpha

% input position limits
min_ail1 = -15.0; %-21.5
max_ail1 =  15.0;  
min_ail2 = -15.0;
max_ail2 =  15.0;  
min_rud  = -15.0; %-30.0;
max_rud  =  15.0;  

% input rate limits
min_ail1_dot = -80.0;    % [deg/s]
max_ail1_dot =  80.0;    % [deg/s]     
min_ail2_dot = -80.0;    % [deg/s]
max_ail2_dot =  80.0;    % [deg/s]     
min_rud_dot  = -120.0;   % [deg/s]
max_rud_dot  =  120.0;   % [deg/s]     

% Controller Reconfiguration
% if min_ail1 == max_ail1 || min_ail2 == max_ail2,
%     
%     % make secondary state objectives less important 
%     Q_R(1,1)= 0.1*Q_R(1,1);    % beta
%     Q_R(3,3)= 0.1*Q_R(3,3);    % roll rate
%     Q_R(4,4)= 0.1*Q_R(4,4);    % yaw rate
%    
%     % make redundant control surface work harder 
%     if min_ail1 == max_ail1 && min_ail2 < max_ail2,
%         Q_R(6,6)= 0.1*Q_R(6,6); % uC_ail2 
%     elseif min_ail2 == max_ail2 && min_ail1 < max_ail1,
%         Q_R(5,5)= 0.1*Q_R(5,5); % uC_ail1
%     end
%     
%     % NB! increase horizon, if computational power is not a constraint    
% end

% if min_rud == max_rud,
%     
%     % make secondary state objectives less important 
% %     Q_R(1,1)= 0.1*Q_R(1,1);    % beta
%     Q_R(3,3)= 0.1*Q_R(3,3);    % roll rate
%     Q_R(4,4)= 0.1*Q_R(4,4);    % yaw rate
%    
%     % make redundant control surface work harder 
%         Q_R(5,5)= 0.1*Q_R(5,5); % uC_ail1
%         Q_R(6,6)= 0.1*Q_R(6,6); % uC_ail2 
%     
%     % NB! increase horizon, if computational power is not a constraint    
% end


% run acado NMPC algorithm
la_out = roll_NMPC_reconfig_RUN(t0, T_h, N, xu0, ref, refT, Q_R, p0, S, sRef,...
         min_ail1, max_ail1, min_ail2, max_ail2, min_rud, max_rud, ... 
         min_ail1_dot, max_ail1_dot, min_ail2_dot, max_ail2_dot,... 
         min_rud_dot, max_rud_dot, ...
         p0_qbar, p0_alpha, p0_Vt,p0_theta, p0_ft, p0_X_tot, p0_Z_tot);
    
  u_ail1 = la_out.U(1);
  u_ail2 = la_out.U(2);
  u_rud = la_out.U(3);
  p_qbar = la_out.P(1);
  p_alpha = la_out.P(2);
  p_Vt = la_out.P(3);
  p_theta = la_out.P(4);
  p_ft = la_out.P(5);
  p_X_tot = la_out.P(6);
  p_Z_tot = la_out.P(7);
  
  fnc_out = [u_ail1, u_ail2, u_rud, p_qbar, p_alpha, p_Vt, p_theta, p_ft, p_X_tot, p_Z_tot]';

end



